/**
 * WEBLAB: Service oriented integration platform for media mining and intelligence applications
 * 
 * Copyright (C) 2004 - 2012 Cassidian, an EADS company
 * 
 * This library is free software; you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License along with this
 * library; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301 USA
 */
package org.ow2.weblab.services;

import java.net.URL;
import java.util.HashMap;
import java.util.Map;

import javax.xml.namespace.QName;

import org.ow2.weblab.core.extended.util.ServiceUtil;
import org.ow2.weblab.core.services.Analyser;
import org.ow2.weblab.core.services.Analyser_Service;
import org.ow2.weblab.core.services.DataOperator;
import org.ow2.weblab.core.services.DataOperator_Service;
import org.ow2.weblab.core.services.Indexer;
import org.ow2.weblab.core.services.Indexer_Service;
import org.ow2.weblab.core.services.ResourceContainer;
import org.ow2.weblab.core.services.ResourceContainer_Service;
import org.ow2.weblab.core.services.Searcher;
import org.ow2.weblab.core.services.Searcher_Service;


/**
 * This class will manage services creation and retrieval
 * From a configuration file
 * @author asaval
 * @deprecated use weblab-client instead (org.ow2.weblab.components.weblab-client)
 */
public class ServicesManager extends BeansManager {
	
	public static enum WebLabServiceInterface {Analyser, Indexer, ResourceContainer, Searcher};
	
	private static final String DEFAULT_CONFIGURATION_FILE = "./services_configuration.xml";

	private Map<String,ServiceDescription> services;
	
	public Map<String, ServiceDescription> getServices() {
		return services;
	}

	public void setServices(Map<String, ServiceDescription> services) {
		this.services = services;
	}

	public static ServicesManager getInstance(String path){
		BeansManager bm = BeansManager.getInstance(path);
		ServicesManager instance = null;
		try{
			instance = bm.get("servicesManager",ServicesManager.class);
		}catch (Exception e) {
			e.printStackTrace();
			// creates an empty instance
			instance = new ServicesManager();
			instance.services = new HashMap<String, ServiceDescription>();
		}
		
		return instance;
	}
	
	public static ServicesManager getInstance(){
		return ServicesManager.getInstance(DEFAULT_CONFIGURATION_FILE);
	}
	
	private Object loadWebLabService(String id, WebLabServiceInterface wi){
		ServiceDescription sd = services.get(id);
		if (sd == null){
			String configurationFile = pathToConfiguration == null ? DEFAULT_CONFIGURATION_FILE:pathToConfiguration;
			throw new IllegalArgumentException(id+" service not found in confuguration, check "+configurationFile);
		}
		
		Object service = sd.getService();
		
		if (service != null){
			return service;
		}
		
		ClassLoader cl = this.getClass().getClassLoader();
		URL configureUrl = cl.getResource("services/WebLab.wsdl");
		
		switch (wi) {
		case Analyser:
			Analyser_Service confServiceA = new Analyser_Service(configureUrl,
					new QName("http://weblab.ow2.org/core/1.2/services", "Analyser"));		
			service = confServiceA.getAnalyserPort();
			break;
		case Indexer:
			Indexer_Service confServiceIndex = new Indexer_Service(configureUrl,
					new QName("http://weblab.ow2.org/core/1.2/services", "Indexer"));		
			service = confServiceIndex.getIndexerPort();
			break;
		case Searcher:
			Searcher_Service confServiceSearch = new Searcher_Service(configureUrl, 
					new QName("http://weblab.ow2.org/core/1.2/services", "Searcher"));		
			service = confServiceSearch.getSearcherPort();
			break;
		case ResourceContainer:
			ResourceContainer_Service confServiceRC = new ResourceContainer_Service(configureUrl, 
					new QName("http://weblab.ow2.org/core/1.2/services", "ResourceContainer"));		
			service = confServiceRC.getResourceContainerPort();			
			break;
		default:
			break;
		}
		ServiceUtil.setEndpointAddress(service, sd.getEndPoint(), "");
		
		sd.setService(service);
		
		return service;
	}
	
	public ResourceContainer getResourceContainer(String id){
		return (ResourceContainer)loadWebLabService(id, WebLabServiceInterface.ResourceContainer);
	}
	
	public Analyser getAnalyser(String id){
		return (Analyser)loadWebLabService(id, WebLabServiceInterface.Analyser);
	}
	
	public Searcher getSearcher(String id){
		return (Searcher)loadWebLabService(id, WebLabServiceInterface.Searcher);
	}
	
	public Indexer getIndexer(String id){
		return (Indexer)loadWebLabService(id, WebLabServiceInterface.Indexer);
	}
	
		
	public DataOperator getDataOperator(String id){
		ServiceDescription sd = services.get(id);
		if (sd == null){
			String configurationFile = this.pathToConfiguration == null ? DEFAULT_CONFIGURATION_FILE:pathToConfiguration;
			throw new IllegalArgumentException(id+" service not found in confuguration, check "+configurationFile);
		}
		Object o = sd.getService();
		if (o == null){
			// we load the service for the first time
			ClassLoader cl = this.getClass().getClassLoader();
			URL configureUrl = cl.getResource("services/DataOperator.wsdl");
			DataOperator_Service confService = new DataOperator_Service(configureUrl, new QName("http://weblab.ow2.org/core/1.2/services/dataoperator", "DataOperator"));		
			o = confService.getDataOperatorPort();
			ServiceUtil.setEndpointAddress(o, sd.getEndPoint(), "");
		}
		return (DataOperator)o;
	}
	
}
